package xyz.maxwellj.chookspeak 

import java.io.File

fun splitList(input: String): List<String> {
    val result = mutableListOf<String>()
    var currentToken = StringBuilder()
    var insideQuotes = false
    
    for (char in input) {
        when {
            char == '"' -> {
                currentToken.append(char)
                insideQuotes = !insideQuotes
            }
            char.isWhitespace() && !insideQuotes -> {
                if (currentToken.isNotEmpty()) {
                    result.add(currentToken.toString())
                    currentToken = StringBuilder()
                }
            }
            else -> currentToken.append(char)
        }
    }
    
    if (currentToken.isNotEmpty()) {
        result.add(currentToken.toString())
    }
    
    return result
}

object files {
    val outputIndexJS = File("out/index.js")
    val outputMessageJS = File("out/message.js}")
    val outputHTML = File("out/index.html") 
    val generatedIndexJS = mutableListOf<String>()
    val generatedMessageJS = mutableListOf<String>()
    val generatedHTML = mutableListOf<String>()
    
    fun write() {
        outputIndexJS.writeText("")
        outputMessageJS.writeText("")
        outputHTML.writeText("")
 
        for (item in generatedHTML) {
            outputHTML.appendText("$item\n")
        }
        for (item in generatedMessageJS) {
            outputMessageJS.appendText("$item\n")
        }
        for (item in generatedIndexJS) {
            outputIndexJS.appendText("$item\n")
        }
    }
}

fun testValidity(input: List<String>, num: Int): Int {
    println("Testing validity of ${input.toString()}...")
    if (input[1] != "=") {
        println("""Error at line $num: Define your variable with "=", not "${input[1]}"""")
        return 1
    }
    return 0
}

fun main(args: Array<String>) {
    println("Chookspeak Language Processor")
    println("Let's Go!")
    var fileLocation: File
    try {
        fileLocation = File(args[0])
    } catch (e: Exception) {
        println("Please specify a file")
        return
    }
    if (!fileLocation.exists()) {
        println("Your Chookspeak file does not exist.")
        return
    }
    val file = fileLocation.readLines()
    var lineCounter = 1
    var commenting = 0
    var eggName = ""
    val status = mutableListOf<String>()
    val variables = mutableListOf<String>()
    for (line in file) {
        val lineArgsUnfiltered = splitList(line) 
        val lineArgs = lineArgsUnfiltered.filterNot { it == "" }
        if (lineArgs.size == 0) {
            continue
        }
        println("line ${lineArgs.toString()}")
        var slc = 0
        for (item in lineArgs) {
            if (item == "//") {
                commenting = 1
            } else if (item == "/*") {
                commenting = 2
                slc = 1
            } else if (item == "*/" && commenting == 2 && slc == 0) {
                commenting = 0
            }
        }
        if (commenting == 0) {
            if (lineCounter == 1) {
                if (lineArgs[0] != "egg") {
                    println("Please specify your egg on your first line of code.")
                    return
                } else {
                    eggName = lineArgs[1]
                    files.generatedIndexJS.add("function egg$eggName() {")                
                    files.generatedHTML.add("""<div id="$eggName" style="display: hidden">""")
                }
            }
            if (lineArgs[0] == "}") {
                try {
                    status.removeLast()
                } catch (e: Exception) {}
                println("status ${status.toString()}")
            }
            if (lineArgs.size > 1) {
                if (lineArgs[1] == "{") {
                    status.add(lineArgs[0])
                    println("status ${status.toString()}")
                }
            }
            if (status == listOf("init")) {
                if (lineArgs[0] == "log") {
                    files.generatedIndexJS.add("console.log(${lineArgs[1]});")
                }
                if (lineArgs[0] == "sendMessage") {
                    files.generatedIndexJS.add("""const egg${eggName}Message = { "type" = "egg-$eggName", "username" = username, "token" = md5(password), "content": ${lineArgs[1]}};${""}""")
                    files.generatedIndexJS.add("ws.send(JSON.stringify(egg${eggName}Message))")
                }
                if (lineArgs[0] == "val") {
                    if (lineArgs[2] == "=") {
                        files.generatedIndexJS.add("const ${lineArgs[1]} = ${lineArgs[3]};")
                    } else {
                        println("Error on line $lineCounter: Define your value with '=', not ${lineArgs[2]}")
                    }
                }
                if (lineArgs[0] == "var") {
                    if (lineArgs[2] == "=") {
                        testValidity(lineArgs, lineCounter)
                        files.generatedIndexJS.add("let ${lineArgs[1]} = ${lineArgs[3]};")
                        variables.add(lineArgs[1])    
                    } else {
                        println("Error on line $lineCounter: Define your variable with '=', not ${lineArgs[2]}")
                    }
                }
                if (variables.contains(lineArgs[0])) {
                    if (lineArgs[1] == "=") {
                        //if ()
                    }
                }
            }
            if (status == listOf("init", "html") && lineArgs[0] != "html") {
                files.generatedHTML.add("${lineArgs.joinToString(" ")}")
            }
            if (status == listOf("init", "js") && lineArgs[0] != "js") {
                files.generatedIndexJS.add("${lineArgs.joinToString(" ")}")
            }           
            lineCounter ++
        } else if (commenting == 1) {
            commenting = 0
        } else if (commenting == 2) {
            commenting = 0
        }
    }
    files.generatedIndexJS.add("}")
    files.generatedHTML.add("</div>")
    files.write()
}
